<?php

namespace Drupal\cite\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Represents an Content Citation Mapping.
 *
 * @ConfigEntityType(
 *   id = "cite_map",
 *   label = @Translation("Content Citation Mapping"),
 *   label_collection = @Translation("Content Citation Mappings"),
 *   label_singular = @Translation("cite as map"),
 *   label_plural = @Translation("cite as maps"),
 *   label_count = @PluralTranslation(
 *     singular = "@count cite as map",
 *     plural = "@count cite as maps",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\cite\ContentCitationMapListBuilder",
 *     "form" = {
 *       "add" = "Drupal\cite\Form\ContentCitationMapCreateForm",
 *       "edit" = "Drupal\cite\Form\ContentCitationMapEditForm",
 *       "delete" = "Drupal\cite\Form\ContentCitationMapDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider"
 *     },
 *   },
 *   config_prefix = "cite",
 *   admin_permission = "administer cite",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "target_entity_type_id",
 *     "target_bundle",
 *     "reference_type",
 *   },
 *   links = {
 *     "add-form" = "/admin/config/bibcite/cite/add",
 *     "edit-form" = "/admin/config/bibcite/cite/{cite_map}",
 *     "delete-form" = "/admin/config/bibcite/cite/{cite_map}/delete",
 *     "collection" = "/admin/config/bibcite/cite"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "target_entity_type_id",
 *     "target_bundle",
 *     "reference_type",
 *     "field_map",
 *   }
 * )
 */
class ContentCitationMap extends ConfigEntityBase {

  /**
   * Unique machine name of the cite map.
   *
   * @var string
   */
  protected $id;

  /**
   * The content citation mapping label.
   *
   * @var string
   */
  protected $label;

  /**
   * The target entity type.
   *
   * @var string
   */
  protected $target_entity_type_id;

  /**
   * The target bundle.
   *
   * @var string
   */
  protected $target_bundle;

  /**
   * The citation reference type.
   *
   * @var string
   */
  protected $reference_type;

  /**
   * The field map.
   *
   * @var array
   */
  protected $field_map = [];

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->label;
  }

  /**
   * Get field mapping.
   *
   * @return array
   *   A field map, content field machine name to reference field machine name.
   */
  public function getFieldMap() {
    return $this->field_map;
  }

  /**
   * Get citation reference field machine name by target content field machine name.
   *
   * @param string $content_field_name
   *   The machine name of a field on the target content entity bundle.
   *
   * @return array
   *   The machine name of the citation reference field.
   */
  public function getMappedField($content_field_name) {
    return $this->field_map[$content_field_name];
  }

  /**
   * Update a content field's cite as field.
   *
   * @param string $content_field_name
   *   A field machine name on the target content entity bundle.
   * @param string $reference_field_name
   *   A field machine name on the citation reference.
   */
  public function updateMappedField($content_field_name, string $reference_field_name) {
    $this->field_map[$content_field_name] = $reference_field_name;
  }

  /**
   * Set field mapping.
   *
   * @param array $field_map
   *   A field map, content field machine name to reference field machine name.
   */
  public function setFieldMap(array $field_map) {
    $this->field_map = $field_map;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetEntityTypeId() {
    return $this->target_entity_type_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetEntityType() {
    return \Drupal::service('entity_type.manager')->getDefinition($this->target_entity_type_id);
  }

  /**
   * {@inheritdoc}
   */
  public function setTargetEntityTypeId($entity_type_id) {
    $this->target_entity_type_id = $entity_type_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getTargetBundle() {
    return $this->target_bundle;
  }

  /**
   * {@inheritdoc}
   */
  public function setTargetBundle($bundle) {
    $this->target_bundle = $bundle;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferenceType() {
    return $this->reference_type;
  }

  /**
   * {@inheritdoc}
   */
  public function setReferenceType($reference_type) {
    $this->reference_type = $reference_type;
  }

}
