<?php

namespace Drupal\cite\Form;

use Drupal\cite\ContentCitationMapInformation;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ContentCitationMapCreateForm.
 *
 * The Content CitationMap create form.
 *
 * @package Drupal\cite\Form
 */
class ContentCitationMapCreateForm extends EntityForm {

  /**
   * The content citation mapping information service.
   *
   * @var \Drupal\cite\ContentCitationMapInformation
   */
  protected $citeMapInfo;

  /**
   * Constructs a ContentCitationMapCreateForm object.
   *
   * @param \Drupal\cite\ContentCitationMapInformation $cite_map_info
   *   The content citation mapping information service.
   */
  public function __construct(ContentCitationMapInformation $cite_map_info) {
    $this->citeMapInfo = $cite_map_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cite.content_citation_mapping_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $form['target_entity_type_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity type'),
      '#default_value' => 'node',
      '#options' => $this->getContentEntityTypeOptions(),
      '#description' => $this->t('Choose the entity type that this citation mapping is for.'),
    ];

    $form['target_bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundle'),
      '#default_value' => '',
      '#options' => $this->getContentBundleOptions(),
      '#description' => $this->t('Choose the bundle (for example, content type) that this citation mapping is for.'),
    ];

    $form['reference_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Reference type'),
      '#default_value' => 'web_article',
      '#options' => $this->getReferenceTypeOptions(),
      '#description' => $this->t('Choose the Bibliography reference type that the citeable content will be mapped to.'),
    ];

    return $form;
  }

  /**
   * Get all the supported content entity types.
   *
   * @return array
   *   Array of content entity types options, keyed by entity type id.
   */
  private function getContentEntityTypeOptions() {
    $entity_types = $this->entityTypeManager->getDefinitions();
    $content_entity_types = array_filter($entity_types, function (EntityTypeInterface $entity_type) {
      return $this->citeMapInfo->isSupportedEntityType($entity_type);
    });
    $options = [];
    foreach ($content_entity_types as $entity_type) {
      $options[$entity_type->id()] = $entity_type->getLabel();
    }

    return $options;
  }

  /**
   * Get all the supported content entity types.
   *
   * @return array
   *   Array of content entity types options, keyed by entity type id.
   */
  private function getContentBundleOptions() {
    // @TODO support more than nodes when we open that up in isSupportedEntityType.
    $options = [];
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    foreach ($types as $type => $info) {
      $options[$type] = $info->label(); // this is more distracting than helpful i think . ' (' . $type . ')';
    }
    return $options;
  }

  /**
   * Get array of Reference types options.
   *
   * @return array
   *   Array of types options.
   */
  protected function getReferenceTypeOptions() {
    $storage = $this->entityTypeManager->getStorage('bibcite_reference_type');
    $reference_types = $storage->loadMultiple();

    return array_map(function ($entity) {
      /** @var \Drupal\bibcite_entity\Entity\ReferenceTypeInterface $entity */
      return $entity->label();
    }, $reference_types);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\cite\Entity\ContentCitationMap $entity */
    $entity = $this->entity;

    $entity->set('id', $entity->getTargetEntityTypeId() . '_' . $entity->getTargetBundle() . '_' . $entity->getReferenceType());
    $entity->set('label', 'Mapping for ' . $entity->getTargetEntityTypeId() . ' ' . $entity->getTargetBundle() . ' to reference type ' . $entity->getReferenceType());

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the @label Content Citation mapping.', [
          '@label' => $entity->label(),
        ]));
        break;
    }
    $form_state->setRedirectUrl($entity->toUrl('edit-form'));
    return $status;
  }

}
