<?php

namespace Drupal\cite\Form;

use Drupal\cite\ContentCitationMapInformation;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ContentCitationMapEditForm
 *
 * The content citation mapping edit form.
 *
 * @package Drupal\cite\Form
 */
class ContentCitationMapEditForm extends EntityForm {

  /**
   * The citation map information service.
   *
   * @var \Drupal\cite\ContentCitationMapInformation
   */
  protected $citeMapInfo;

  /**
   * Constructs a ContentCitationMapEditForm object.
   *
   * @param \Drupal\cite\ContentCitationMapInformation $cite_map_info
   *   The content citation mapping information service.
   */
  public function __construct(ContentCitationMapInformation $cite_map_info) {
    $this->citeMapInfo = $cite_map_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cite.content_citation_mapping_info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    /** @var \Drupal\cite\Entity\ContentCitationMap $entity */
    $entity = $this->entity;

    $form['#title'] = $this->t('Configure %label', ['%label' => $entity->label()]);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#disabled' => !$entity->isNew(),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\cite\Entity\ContentCitationMap::load',
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['field_map'] = [
      '#type' => 'table',
      '#caption' => $this->t('Fields mapping'),
      '#header' => [$this->t('Reference field'), $this->t('Content field')],
    ];

    $reference_fields = $this->citeMapInfo->getReferenceFieldOptions($entity->getReferenceType());
    $content_field_options = $this->citeMapInfo->getContentFieldOptions($entity->getTargetEntityTypeId(), $entity->getTargetBundle());
    $field_defaults = $entity->getFieldMap();
    foreach ($reference_fields as $reference_field => $reference_field_label) {
      $form['field_map'][$reference_field]['reference_field_description'] = [
        '#type' => 'item',
        '#markup' => $reference_field_label,
        '#description' => $reference_field,
      ];
      $form['field_map'][$reference_field]['mapped_content_field'] = [
        '#type' => 'select',
        '#options' => $content_field_options,
        '#empty_option' => $this->t('- Select -'),
        '#default_value' => $field_defaults[$reference_field] ?? NULL,
      ];
    }

//      $form['access_rule_container']['help'] = [
//        '#type' => 'markup',
//        '#markup' => '<p>' . $this->t('Access rules allow you to define fine-grained access controls based on field and environmental values.') . '</p>',
//      ];

    // $entity->getTargetEntityTypeId()
    // $entity_type = $this->entity->getTargetEntityType();

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\cite\Entity\ContentCitationMap $entity */
    $entity = $this->entity;
    $values = $form_state->getValues();
    $field_map = [];
    foreach ($values['field_map'] as $reference_field => $array) {
      // Keep our config clean and only save if there's a value mapped.
      if ($content_field = $array['mapped_content_field']) {
        $field_map[$reference_field] = $content_field;
      }
    }
    $entity->setFieldMap($field_map);

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        // This has always happened already on the previous step, but tradition.
        $this->messenger()->addMessage($this->t('Created the @label.', [
          '@label' => $entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the @label.', [
          '@label' => $entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($entity->toUrl('collection'));

    return $status;
  }

  /**
   * Provides the edit title.
   *
   * Note that this only affects the breadcrumb. The main title is set in the
   * form itself.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return string|null
   *   The title for the entity edit page, if an entity was found.
   */
  public function getTitle(RouteMatchInterface $route_match) {
    $entity = $route_match->getParameter('cite_map');
    return $this->t('%label', ['%label' => $entity->label()]);
  }

}
